function [coord, in]=hcfft_coordhc(d, n, in)
%
% Computes the coordinates of a hyperbolic cross of dimension d and level n
% in the order of the indexing function.
%
% [coord in]=hcfft_coordhc(d, n, in)
%
% d     = dimension
% n     = level of hc
% in    = object from hcfft_initialize(d,n), optional
%
% Example:
%
%   h=hcfft_coordhc(4,3) % gives Nx4 matrix, rows are hc points
%                        % of dimension 4 and level 3
%
% See also: hcfft_vec2hc, hcfft_hc2vec
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcfft_coordhc.m 536 2009-05-06 13:20:30Z kaemmerer $

%in (struct with some initial matrices) is an optional argument

    if(nargin==2)
        in=hcfft_initialize(d,n);
       % disp('initialization in hcfft_coordhc');
    end
    global HCFFT_coord;
    HCFFT_coord=zeros(in.s(d, n+1),d);
    l=zeros(1,d);
    for k=0:n   
        numl(l, 1, d, k, n, in);
    end
    coord = HCFFT_coord;
    clear HCFFT_coord;
end

function v = sigma(q,d)
    v=zeros(1,d);
    for i=1:d
        if(mod(q(i),2)==0)
            v(i)=-q(i)/2;
        else
            v(i)=(q(i)+1)/2;
        end 
    end
end

function numl(l, d1, d, k, n, in)
    m=0;
    if(d1>1)
       m=sum(l(1:d1-1));
    end
    for i=0:k-m
        l(d1)=i;
        if(d1<d-1)
            numl(l, d1+1, d, k, n, in);
        else
            l(d)=k-m-i;
            q=zeros(1,d);
            numq(q, q, 1, d, l, n, k, in);
        end 
    end
end

function numq(q, q1, d1, d, l, n, k, in)
    global HCFFT_coord;
    for i=1:ceil(2^(l(d1)-1))
        q1(d1)=i;
        if(l(d1)==0)
            q(d1)=0;
        else
            q(d1)=2*i-1;
        end
        if(d1<d)
            numq(q, q1, d1+1, d, l, n, k, in);
        else
            ind=hcfft_index(l, q, d, k, in);
            HCFFT_coord(ind,:)=sigma(floor(2.^(l-1))+q1-1, d);
        end
    end            
end
