% Script hcfft_times.m; Example for the paper
% Nonequispaced hyperbolic cross fast Fourier transform
%
% Computational time for the HCDFT and HCFFT with respect to the refinement and
% the dimension, produces Figure 5.1
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcfft_times.m 544 2009-05-13 09:17:01Z kaemmerer $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figure 5.1(a-c), fixed d, increasing n
dd=[2,3,10];
nn_max=[20,17,11];
hcdft_max=[12,11,7];
matrix_max=[12,11,6];
print_name={'test_hcfft_2d_e.png','test_hcfft_3d_e.png','test_hcfft_10d_e.png'};

for i=1:3

  d=dd(i);
  n_max=nn_max(i);;

  t_hcfft = inf*ones(n_max,1);
  t_hcdft1 = inf*ones(n_max,1);
  t_hcdft2 = inf*ones(n_max,1);

  disp(sprintf('\nComputational times of the HCFFT for d=%d',d));
  disp(sprintf('n\tt_hcfft\tt_hcdft\tt_matrix'));
  for n=1:n_max
    in = hcfft_initialize(d, n);
    N=in.s(d, n+1);
    f_hat=randn(N,1)+i*randn(N,1);
    f=hcfft(f_hat,'notransp','noinv',in);

    t=cputime;
    f=hcfft(f_hat,'notransp','noinv',in);
    t_hcfft(n)=cputime-t;

    if(n<=hcdft_max(i))
      t=cputime;
      f=hcdft(f_hat,'notransp',in);
      t_hcdft1(n)=cputime-t;
    end;

    if(n<=matrix_max(i))
      k=hcfft_coordhc(d, n, in);
      x=hcfft_coordsg(d, n, in);
      F=exp(2*pi*i*x*k');
      t=cputime;
      f=F*f_hat;
      t_hcdft2(n)=cputime-t;
    end;
    disp(sprintf('%d\t%1.1e\t%1.1e\t%1.1e',n,t_hcfft(n),t_hcdft1(n),t_hcdft2(n)));
  end;

  save(print_name{i}, 't_hcfft t_hcdft1 t_hcdft2 n_max')
  figure(1);
  h=semilogy(1:length(t_hcfft),t_hcfft,'-+k',...
             1:length(t_hcdft1),t_hcdft1,'--sk',...
             1:length(t_hcdft2),t_hcdft2,'-.dk');
  set(h,'LineWidth',2); set(h,'MarkerSize',8); set(gca,'FontSize',15);

  legend('HCFFT', 'HCDFT (direct summation)', 'HCDFT (matrix-vector)','Location','NorthWest');
  xlabel('Refinement n');
  ylabel('Computational time in seconds');
  axis([1,n_max,10^-2,10^4]);

  print('-dpng',print_name{i});
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figure 5.1(d), fixed n, increasing d
n=6;
d_max=20;

hcfftdd = inf*ones(d_max,1);
hcdftdd1 = inf*ones(d_max,1);
hcdftdd2 = inf*ones(d_max,1);

for d=2:d_max
  disp(sprintf('Spatial dimension d=%d',d));
  in = hcfft_initialize(d, n);
  N=in.s(d, n+1);
  f_hat=randn(N,1)+i*randn(N,1);
  f=hcfft(f_hat,'notransp','noinv',in);

  t=cputime;
  f=hcfft(f_hat,'notransp','noinv',in);
  hcfftdd(d)=cputime-t;

  if(d<=12)
    t=cputime;
    f=hcdft(f_hat,'notransp','loop',in);
    hcdftdd1(d)=cputime-t;
  end;

  if(d<=10)
    k=hcfft_coordhc(d, n, in);
    x=hcfft_coordsg(d, n, in);
    F=exp(2*pi*i*x*k');
    t=cputime;
    f=F*f_hat;
    hcdftdd2(d)=cputime-t;
    clear('F');
  end;
end;

save('test_hcfft_dd_e','hcfftdd hcfftdd1 hcfftdd2');
figure(4);
h=semilogy(2:(length(hcfftdd)+1),hcfftdd,'-+k',...
	   2:(length(hcdftdd1)+1),hcdftdd1,'--sk',...
	   2:(length(hcdftdd2)+1),hcdftdd2,'-.dk');
set(h,'LineWidth',2); set(h,'MarkerSize',8); set(gca,'FontSize',15);

legend('HCFFT', 'HCDFT (direct summation)', 'HCDFT (matrix-vector)','Location','NorthWest');
xlabel('Dimension d');
ylabel('Computational time in seconds');
axis([2,20,10^-2,10^4]);

print test_hcfft_dd_e.png -dpng
