function [A in]=hcfft_vec2hc(B, in)
% Converts hc vector B to hc sparse tensor A.
% B is a vector containing a hyperbolic cross of dimension d and level n.
%
% [A in] = hcfft_vec2hc(B, in)
%
% B     = vector containing fourier voefficients over hyperbolic cross
% in    = struct from hcfft_initialize(d,n) or simple vector [d,n]
%         with d dimension and n level of hyperbolic cross vector B
%
% Example:
%   see <a href="matlab:help hcfft_hc2vec">hcfft_hc2vec</a>
%
% See also: hcfft_hc2vec, hcfft_coordhc
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcfft_vec2hc.m 544 2009-05-13 09:17:01Z kaemmerer $

    if(length(in)==1)
        [d,n]=size(in.s);
        n=n-1;
    elseif(length(in)==2)
        d=in(1); n=in(2);
        in=hcfft_initialize(d,n);
    else
        disp('problem with argument ''in''');
        return;
    end

    if(length(B)~=in.s(d, n+1))
        disp('Length of input vector doesn''t fit to given dimension d and level n');
        return;
    end
    global HCFFT_B HCFFT_coord HCFFT_val;
    HCFFT_B=B;
    HCFFT_coord=zeros(in.s(d, n+1),d);
    HCFFT_val=zeros(in.s(d, n+1),1);
    l=zeros(1,d);
    for k=0:n   
        numl(l, 1, d, k, n, in);
    end
    A=sptensor(HCFFT_coord, HCFFT_val);
    clear HCFFT_B HCFFT_coord HCFFT_val;
end

function v = sigma(q,d)
    v=zeros(1,d);
    for i=1:d
        if(mod(q(i),2)==0)
            v(i)=-q(i)/2;
        else
            v(i)=(q(i)+1)/2;
        end 
    end
end

function numl(l, d1, d, k, n, in)
    m=0;
    if(d1>1)
       m=sum(l(1:d1-1));
    end
    for i=0:k-m
        l(d1)=i;
        if(d1<d-1)
            numl(l, d1+1, d, k, n, in);
        else
            l(d)=k-m-i;
            q=zeros(1,d);
            numq(q, q, 1, d, l, n, k, in);
        end 
    end
end

function numq(q, q1, d1, d, l, n, k, in)
    global HCFFT_B HCFFT_coord HCFFT_val;
    for i=1:ceil(2^(l(d1)-1))
        q1(d1)=i;
        if(l(d1)==0)
            q(d1)=0;
        else
            q(d1)=2*i-1;
        end
        if(d1<d)
            numq(q, q1, d1+1, d, l, n, k, in);
        else
            ind=hcfft_index(l, q, d, k, in);
            HCFFT_coord(ind,:)=sigma(floor(2.^(l-1))+q1-1, d)+2^(n-1);
            HCFFT_val(ind)=HCFFT_B(ind);
        end
    end            
end
