function [C in]=hcfft_vec2sg(B, in)
% Converts sg vector B to sg sparse tensor C.
% B is a vector containing a sparse grid of dimension d and level n.
%
% [C in] = hcfft_vec2sg(B, in)
%
% B     = vector containing a sparse grid
% in    = struct from hcfft_initialize(d,n) or simple vector [d,n]
%         d dimension and n level of sparse grid vector B
%
% Example:
%   see <a href="matlab:help hcfft_sg2vec">hcfft_sg2vec</a>
%
% See also: hcfft_sg2vec, hcfft_coordsg
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcfft_vec2sg.m 544 2009-05-13 09:17:01Z kaemmerer $


    if(length(in)==1)
        [d,n]=size(in.s); n=n-1;
    elseif(length(in)==2)
        d=in(1); n=in(2);
        in=hcfft_initialize(d,n);
    else
        disp('problem with argument ''in'''); return;
    end

    if(length(B)~=in.s(d, n+1))
        disp('Length of input matrix doesn''t fit to given dimension d and level n');
        return;
    end
    l=zeros(1,d);
    global HCFFT_coord HCFFT_val HCFFT_B;
    HCFFT_coord=zeros(in.s(d, n+1),d);
    HCFFT_val=zeros(in.s(d, n+1),1);
    HCFFT_B=B;
    for k=0:n
        numl(l, 1, d, k, n, in);
    end
    C=sptensor(HCFFT_coord, HCFFT_val);
    clear HCFFT_coord HCFFT_val HCFFT_B;
end

function v = pos(l,q,n,d)
    % gibt Position des Punkts (l,q) in der (vollen) Matrix im Zeitbereich zurck
    v=zeros(1,d);
    for i=1:d
        v(i)=2^(n-l(i))*q(i)+1;
    end
end

function numl(l, d1, d, k, n, in)
    m=0;
    if(d1>1)
       m=sum(l(1:d1-1));
    end
    for i=0:k-m
        l(d1)=i;
        if(d1<d-1)
            numl(l, d1+1, d, k, n, in);
        else
            l(d)=k-m-i;
            q=zeros(1,d);
            numq(q, 1, d, l, n, k, in);
        end 
    end
end

function numq(q, d1, d, l, n, k, in)
    global HCFFT_coord HCFFT_val HCFFT_B;
    for i=1:ceil(2^(l(d1)-1))
        if(l(d1)==0)
            q(d1)=0;
        else
            q(d1)=2*i-1;
        end
        if(d1<d)
            numq(q, d1+1, d, l, n, k, in);
        else
            ind=hcfft_index(l, q, d, k, in);
            HCFFT_coord(ind,:)=pos(l, q, n, d);
            HCFFT_val(ind)=HCFFT_B(ind);
        end
    end            
end
